@@ -27,10 +27,13 @@
 
 '''A literate programming extension for Sphinx'''
 
-__version__ = '0.1.1'
+__version__ = '0.1.2'
 
+from hashlib import sha256
+import html
 import io
 import os
+import posixpath
 import re
 from typing import Any, Iterator
 
@@ -116,7 +119,7 @@ class TangleBuilder(Builder):
         tree = self.env.get_doctree(master)
         tree = inline_all_toctrees(self, set(), master, tree, darkgreen, [master])
         return tree
-    def write(self, *ignored: any) -> None:
+    def write(self, *ignored: Any) -> None:
         chunks = {} # dict of chunk name to list of chunks defined by that name
         files = [] # the list of files
     
@@ -136,42 +139,235 @@ class TangleBuilder(Builder):
         unused = {name for name in chunks}
         
         for filename in files:
-            # some basic sanity checking for the file name
-            if '..' in filename or os.path.isabs(filename):
-                raise ExtensionError(
-                    "Chunk name is invalid file name: {}".format(filename),
-                    modname=__name__,
-                )
-            # determine the full path, and make sure the directory exists before
-            # creating the file
-            fullpath = os.path.join(self.outdir, filename)
-            dirname = os.path.dirname(fullpath)
-            if dirname:
-                os.makedirs(dirname, exist_ok=True)
-        
-            unused.discard(filename)
-        
-            with open(fullpath, 'w') as f:
+            with self.writer(filename, unused) as writer:
+                writer.enter_expansion(filename, True)
                 for chunk in chunks[filename]:
+                    writer.enter_chunk(chunk)
                     for line in chunk.astext().splitlines():
-                        _write_line(f, line, chunks, '', '', ldelim, rdelim, unused, [filename])
+                        _write_line(writer, line, chunks, '', '', ldelim, rdelim)
+                    writer.exit_chunk()
+                writer.exit_expansion()
         for chunk_name in unused:
             for chunk in chunks[chunk_name]:
                 logger.warning(
                     '{0.source}:{0.line}: Code chunk "{1}" defined but not used'
                         .format(chunk, chunk_name)
                 )
+    def writer(self, filename: str, unused: set[str]) -> 'TangleWriter':
+        return TangleWriter(filename, self.outdir, unused)
+class TangleWriter:
+    def __init__(
+            self,
+            filename: str,
+            outdir: str,
+            unused: set[str],
+            filename_suffix: str = ''
+    ):
+        self.unused = unused
+        self.path = []
+    
+        # some basic sanity checking for the file name
+        if '..' in filename or os.path.isabs(filename):
+            raise ExtensionError(
+                "Chunk name is invalid file name: {}".format(filename),
+                modname=__name__,
+            )
+        # determine the full path, and make sure the directory exists before
+        # creating the file
+        fullpath = os.path.join(outdir, filename)
+        dirname = os.path.dirname(fullpath)
+        if dirname:
+            os.makedirs(dirname, exist_ok=True)
+    
+        self.filename = filename
+    
+        self.f = open(fullpath + filename_suffix, 'w')
+    def close(self) -> None:
+        self.f.close()
+    def __enter__(self) -> 'TangleWriter':
+        return self
+    
+    def __exit__(self, *ignored: Any) -> None:
+        self.close()
+    def write_line(self, line: str) -> None:
+        self.f.write(line + '\n')
+    
+    def enter_expansion(self, chunk_name: str, is_file: bool = False) -> None:
+        # update bookeeping variables
+        self.unused.discard(chunk_name)
+        if chunk_name in self.path:
+            self.path.append(chunk_name)
+            raise ExtensionError(
+                'Loop found in chunks: {}'.format(' -> '.join(self.path)),
+                modname = __name__,
+            )
+        self.path.append(chunk_name)
+    
+    def exit_expansion(self) -> None:
+        self.path.pop()
+    
+    def enter_chunk(self, chunk_node: nodes.Node) -> None:
+        pass
+    
+    def exit_chunk(self) -> None:
+        pass
+class AnnotatedTangleBuilder(TangleBuilder):
+    name = 'annotated-tangle'
+
+    def writer(self, filename: str, unused: set[str]) -> TangleWriter:
+        return AnnotatedTangleWriter(filename, self.outdir, unused, self)
+
+    def write(self, *opts: Any) -> None:
+        self.max_depth = 1
+
+        super().write(*opts)
+
+        self.write_css()
+
+    def write_css(self) -> None:
+        os.makedirs(os.path.join(self.outdir, '_static'), exist_ok=True)
+        with open(os.path.join(self.outdir, '_static/annotated.css'), 'w') as f:
+            f.write('''
+    html {
+      background: white;
+      color: black;
+    }
+    
+    /* don't indent the first ul, but indent subsequent ones */
+    ul {
+      padding: 0;
+      margin: 0;
+    }
+    
+    ul ul {
+      padding-left: 1.5rem;
+      margin-left: 0;
+    }
+    
+    li {
+      border: 1px solid gray;
+      margin-right: -1px;
+      margin-left: -1px;
+      padding-left: 0.5rem;
+      list-style: none;
+    }
+    
+    /* avoid doubling up borders for adjacent chunks */
+    li + li {
+      border-top: 0px none;
+    }
+    
+    pre {
+      margin: 0.5rem 0 0 0;
+      padding: 0;
+      background: LightGray;
+    }
+    
+    pre .lineno {
+      color: black;
+      display: inline-block;
+      width: 4em;
+      text-align: right;
+      border-right: 3px solid gray;
+      padding-right: 0.5rem;
+      margin-right: 0.5rem;
+      background: white;
+      user-select: none;
+      -webkit-user-select: text;
+      -webkit-user-select: none;
+      -moz-user-select: none;
+      -ms-user-select: none;
+    }
+    
+    /* highlight the target */
+    /* if the target is a line number */
+    pre div:target {
+      background: orange;
+    }
+    
+    /* if the target is a chunk */
+    li:target {
+      border: 3px solid orange;
+      margin-right: -3px;
+      margin-left: -3px;
+    }
+    
+    li:target > .chunkname{
+      font-weight: bold;
+    }
+    
+    li:target pre {
+      background: orange;
+    }
+    ''')
+    
+            for depth in range(0, self.max_depth):
+                f.write('''
+    ul {}pre {{
+      margin-left: {}rem;
+    }}
+    '''.format('ul ' * depth, 2 * (self.max_depth - depth)))
+
+class AnnotatedTangleWriter(TangleWriter):
+    def __init__(
+            self,
+            filename: str,
+            outdir: str,
+            unused: set[str],
+            builder: AnnotatedTangleBuilder,
+    ):
+        super().__init__(filename, outdir, unused, '.html')
+        self.lineno = 1
+        self.node_path = []
+        self.builder = builder
+    def write_line(self, line: str) -> None:
+        self.f.write('<div id="L{0}"><a class="lineno" href="#L{0}">{0}</a>{1}</div>'.format(self.lineno, html.escape(line)))
+        self.lineno = self.lineno + 1
+    def enter_chunk(self, chunk_node: nodes.Node) -> None:
+        super().enter_chunk(chunk_node)
+    
+        self.node_path.append(chunk_node)
+        hash = sha256(':'.join(['{0.source}:{0.line}'.format(c) for c in self.node_path]).encode('utf-8')).hexdigest()
+    
+        chunk_name = chunk_node['code-chunk-name']
+        if 'code-chunk-is-file' in chunk_node:
+            self.f.write('<li id="{}"><span class="chunkname"><code>{}</code></span><pre>'.format(hash, html.escape(chunk_name)))
+        else:
+            self.f.write('<li id="{}"><span class="chunkname">{}</span><pre>'.format(hash, html.escape(chunk_name)))
+    
+    def exit_chunk(self) -> None:
+        self.f.write('</pre></li>')
+        self.node_path.pop()
+        super().exit_chunk()
+    def enter_expansion(self, chunk_name: str, is_file: bool = False) -> None:
+        super().enter_expansion(chunk_name, is_file)
+        if len(self.path) > self.builder.max_depth:
+            self.builder.max_depth = len(self.path)
+    
+        if is_file:
+            self.f.write('<html><head><title>{}</title>'.format(html.escape(chunk_name)))
+            css_file = posixpath.relpath('_static/annotated.css', posixpath.dirname(self.filename))
+            css_file = html.escape(css_file)
+            self.f.write('<link rel="stylesheet" type="text/css" href="{}" />'.format(css_file))
+            self.f.write('</head><body><ul>')
+        else:
+            self.f.write('</pre><ul>')
+    
+    def exit_expansion(self) -> None:
+        if len(self.path) == 1:
+            self.f.write('</ul></body></html>')
+        else:
+            self.f.write('</ul><pre>')
+        super().exit_expansion()
 
 def _write_line(
-        f: io.IOBase,
+        writer: TangleWriter,
         line: str,
         chunks: dict[str, Any],
         prefix: str,
         suffix: str,
         ldelim: str,
         rdelim: str,
-        unused: set[str],
-        path: list[str],
 ) -> None:
     # check if the line contains the left and right delimiter
     s1 = line.split(ldelim, 1)
@@ -180,15 +376,6 @@ def _write_line(
         if len(s2) == 2:
             # delimiters found, so get the chunk name
             chunk_name = s2[0].strip()
-            # update bookeeping variables
-            unused.discard(chunk_name)
-            if chunk_name in path:
-                path.append(chunk_name)
-                raise ExtensionError(
-                    'Loop found in chunks: {}'.format(' -> '.join(path)),
-                    modname = __name__,
-                )
-            path.append(chunk_name)
 
             # write the chunks associated with the name
             try:
@@ -198,31 +385,33 @@ def _write_line(
                     'Unknown chunk name: {}'.format(chunk_name),
                     modname = __name__,
                 )
+            writer.enter_expansion(chunk_name)
             for ins_chunk in ref_chunks:
+                writer.enter_chunk(ins_chunk)
                 for ins_line in ins_chunk.astext().splitlines():
                     # recursively call this function with each line of the
                     # referenced code chunks
                     _write_line(
-                        f,
+                        writer,
                         ins_line,
                         chunks,
                         prefix + s1[0],
                         s2[1] + suffix,
                         ldelim,
                         rdelim,
-                        unused,
-                        path,
                     )
+                writer.exit_chunk()
+            writer.exit_expansion()
 
-            path.pop()
             return
 
     # delimiters not found, so just write the line
-    f.write(prefix + line + suffix + '\n')
+    writer.write_line(prefix + line + suffix)
 def setup(app: Sphinx) -> dict[str, Any]:
     app.add_directive('literate-code', LiterateCode)
 
     app.add_builder(TangleBuilder)
+    app.add_builder(AnnotatedTangleBuilder)
 
     app.add_config_value(
         'literate_delimiters',
